//     Copyright 2017, Kay Hayen, mailto:kay.hayen@gmail.com
//
//     Part of "Nuitka", an optimizing Python compiler that is compatible and
//     integrates with CPython, but also works on its own.
//
//     Licensed under the Apache License, Version 2.0 (the "License");
//     you may not use this file except in compliance with the License.
//     You may obtain a copy of the License at
//
//        http://www.apache.org/licenses/LICENSE-2.0
//
//     Unless required by applicable law or agreed to in writing, software
//     distributed under the License is distributed on an "AS IS" BASIS,
//     WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
//     See the License for the specific language governing permissions and
//     limitations under the License.
//
#ifndef __NUITKA_MAPPINGS_H__
#define __NUITKA_MAPPINGS_H__

NUITKA_MAY_BE_UNUSED static int MAPPING_HAS_ITEM( PyObject *mapping, PyObject *key )
{
    PyObject *result = PyObject_GetItem( mapping, key );

    if ( result == NULL )
    {
        PyObject *error = GET_ERROR_OCCURRED();

        if ( EXCEPTION_MATCH_BOOL_SINGLE( error, PyExc_KeyError ) )
        {
            CLEAR_ERROR_OCCURRED();
            return 0;
        }
        else
        {
            return -1;
        }
    }
    else
    {
        return 1;
    }
}

#endif